package com.uoc.pfc.catalogoweb.service.dao;

import com.uoc.pfc.catalogoweb.service.dao.DAOException;
import com.uoc.pfc.catalogoweb.service.ibatis.IbatisLocator;


/**
 * Implementacion DAO Ibatis
 */
public class IbatisDAO implements DAO {
    
    /**
     * Utilizado para el manejo de transacciones
     */
    private IbatisSqlMapClient sqlMapClient;

    /**
     * Transaccion
     */
    private Object transaction;
 
    private static final String KEY_COMMIT = "Se produjo un error al intentar realizar el commit: {0}";
    private static final String KEY_ROLLBACK = "Se produjo un error ejecutando rollback: {0}";

    /**
     * Inicializamos IbatisSqlMapClient
     */
    public IbatisDAO() {
        sqlMapClient =
            new IbatisSqlMapClient(IbatisLocator.getInstance().getSqlMapClient());
    }
    
    /**
     * Crea un Ibatis DAO con el sqlmap.
     * Si es nulo se inicializa
     * @param sqlMapClient sqlMapclient para hacar las consultas
     */
    public IbatisDAO(IbatisSqlMapClient sqlMapClient){
        this.sqlMapClient = sqlMapClient;
        if (this.sqlMapClient == null){
            this.sqlMapClient = 
                new IbatisSqlMapClient(IbatisLocator.getInstance().getSqlMapClient());
        }
    }
    
    public void open() throws DAOException {
    }
    
    public void open(String aName) throws DAOException {
    }
    
   public void close() throws DAOException {
    }

    /**
     * Abre transaccion
     */
    public void beginTransaction() throws DAOException {
        try {
            sqlMapClient.startTransaction();
            transaction = new Object();
        } catch (Exception e) {
            new DAOException(e);
        }
    }
    
    /**
     * Realiza commit de la transaccion
     */
    public void commitTransaction() throws DAOException {
        try {
            sqlMapClient.commitTransaction();
            transaction = null;
        } catch (Exception e) {
            new DAOException(e, KEY_COMMIT);
        } finally {
            sqlMapClient.endTransaction();
        }
    }
    
    public Object getTransaction() throws DAOException {
        return transaction;
    }
    
    public boolean isInTransaction() throws DAOException {
        return transaction != null;
    }
    
    /**
     * Realiza rollback
     */
    public void rollbackTransaction() throws DAOException {
        try {
            sqlMapClient.endTransaction();
            transaction = null;
        } catch (Exception e) {
            new DAOException(e, KEY_ROLLBACK);
        }
    }
    
    public void setTransaction(Object transaction) throws DAOException {
        this.transaction = transaction;
    }
    
    /**
     * Devuelve el objeto del tipo IbatisSqlMapClient utilizado 
     * para el manejo de las transacciones
     * @return IbatisSqlMapClient
     */
    public IbatisSqlMapClient getSqlMapClient() {
        return sqlMapClient;
    }
    
    /**
     * @param client
     */
    public void setSqlMapClient(IbatisSqlMapClient client) {
        sqlMapClient = client;
    }
    
}
