/*
 * CADESqlMapClient.java
 *
 * Created on 24 de noviembre de 2007, 19:42
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.uoc.pfc.catalogoweb.service.dao;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import com.ibatis.common.util.PaginatedList;
import com.ibatis.sqlmap.client.SqlMapClient;
import com.ibatis.sqlmap.client.SqlMapSession;
import com.ibatis.sqlmap.client.event.RowHandler;


/**
 * Objeto que contiene SqlMapClient utilizado para manejo de transacciones.
 * Necesario para envolver las SQLException en DAOException
 */
public class IbatisSqlMapClient {

    private SqlMapClient sqlMapClient = null;	

    private static final String KEY_COMMIT = "Se produjo un error al intentar realizar el commit: {0}";
    private static final String KEY_ENDTRANSACTION = "Error al cerrar la transaccin {0}";
    private static final String KEY_EXECUTEBATCH = "Error al ejecutar el batch";    
    private static final String KEY_CURRENTCONNECTION = "Error al obtener la conexin actual.";
    private static final String KEY_USERCONNECTION = "Error al establecer la conexin de usuario.";
    private static final String KEY_STARTBATCH = "Error al comenzar un batch.";
    private static final String KEY_BEGINTRANSACTION = "Error al iniciar la transaccin. Posible causa: no se pudo conectar con la base de datos. Conexin: {0}";
    
    public IbatisSqlMapClient (SqlMapClient client){
        sqlMapClient = client;
    }

    /**
     * commitTransaction
     * @throws DAOException
     */
    public void commitTransaction() throws DAOException {
	 
        try {		
            sqlMapClient.commitTransaction();
		} catch(SQLException ex) {
	            throw new DAOException(ex, KEY_COMMIT);
		}
    }

    /**
     * delete
     * @param string String
     * @param object Object
     * @return int
     * @throws DAOException
     */
    public int delete(String string, Object object) throws DAOException {
        try {
            return sqlMapClient.delete(string, object);
        } catch(SQLException ex) {
            throw createException(ex, string, object);
        }
    }

    /**
     * endTransaction
     * @throws DAOException
     */
    public void endTransaction() throws DAOException {
        try {
            sqlMapClient.endTransaction();
        } catch(SQLException ex) {
            throw new DAOException(ex, KEY_ENDTRANSACTION);
        }
    }
        
    /**
     * executeBatch
     * @return int
     * @throws DAOException
     */
    public int executeBatch() throws DAOException {
        try {
            return sqlMapClient.executeBatch();
        } catch(SQLException ex) {
            throw new DAOException(ex, KEY_EXECUTEBATCH);
        }
    }
        
    /**
     * flushDataCache
     * @param string String
     */
    public void flushDataCache(String string) {
        sqlMapClient.flushDataCache(string);
    }
        
    /**
     * flushDataCache
     */
    public void flushDataCache() {
        sqlMapClient.flushDataCache();
    }
        
    /**
     * getCurrentConnection
     * @return Connection
     * @throws DAOException
     */
    public Connection getCurrentConnection() throws DAOException {
        try {
            return sqlMapClient.getCurrentConnection();
        } catch(SQLException ex) {
            throw new DAOException(ex, KEY_CURRENTCONNECTION);
        }
    }
        
    /**
     * getDataSource
     * @return DataSource
     */
    public DataSource getDataSource() {
        return sqlMapClient.getDataSource();
    }
        
    /**
     * insert
     * @param string String
     * @param object Object
     * @return Object
     * @throws DAOException
     */
    public Object insert(String string, Object object) throws DAOException {
        try {
            return sqlMapClient.insert(string , object);
        } catch(SQLException ex) {
            throw createException(ex, string, object);
        }
    }
        
    /**
     * openSession
     * @param connection Connection
     * @return SqlMapSession
     */
    public SqlMapSession openSession(Connection connection) {
        return sqlMapClient.openSession(connection);
    }
        
    /**
     * openSession
     * @return SqlMapSession
     */
    public SqlMapSession openSession() {
        return sqlMapClient.openSession();
    }
        
    /**
     * queryForList
     * @param string String
     * @param object Object
     * @param _int int
     * @param _int3 int
     * @return List
     * @throws DAOException
     */
    public List queryForList(String string, Object object, int _int, int _int3) 
    	throws DAOException {
    	
        try {
            return sqlMapClient.queryForList(string, object, _int, _int3);
        } catch(SQLException ex) {
            throw createException(ex,string,object);
        }
    }
        
    /**
     * queryForList
     * @param string String
     * @param object Object
     * @return List
     * @throws DAOException
     */
    public List queryForList(String string, Object object) 
    	throws DAOException {
    	
        try {
            return sqlMapClient.queryForList(string, object);
        } catch(SQLException ex) {
            throw createException(ex, string, object);
        }
    }
        
    /**
     * queryForMap
     * @param string String
     * @param object Object
     * @param string2 String
     * @param string3 String
     * @return Map
     * @throws DAOException
     */
    public Map queryForMap(String string, Object object, String string2, String string3)
        throws DAOException {
    	
        try {
            return sqlMapClient.queryForMap(string, object, string2, string3);
        } catch(SQLException ex) {
            throw createException(ex, string, object);
        }
    }
        
    /**
     * queryForMap
     * @param string String
     * @param object Object
     * @param string2 String
     * @return Map
     * @throws DAOException
     */
    public Map queryForMap(String string, Object object, String string2)
        throws DAOException {
    	
        try {
            return sqlMapClient.queryForMap(string, object, string2);
        } catch(SQLException ex) {
            throw createException(ex, string, object);
        }
    }
        
    /**
     * queryForObject
     * @param string String
     * @param object Object
     * @param object2 Object
     * @return Object
     * @throws DAOException
     */
    public Object queryForObject(String string, Object object, Object object2)
        throws DAOException {
    	
        try {
            return sqlMapClient.queryForObject(string, object, object2);
        } catch(SQLException ex) {
            throw createException(ex, string, object);
        }
    }
        
    /**
     * queryForObject
     * @param string String
     * @param object Object
     * @return Object
     * @throws DAOException
     */
    public Object queryForObject(String string, Object object)
        throws DAOException {
    	
        try {
            return sqlMapClient.queryForObject(string, object);
        } catch(SQLException ex) {
            throw createException(ex, string, object);
        }
    }

    /**
     * queryForPaginatedList
     * @param string String
     * @param object Object
     * @param _int int
     * @return PaginatedList
     * @throws DAOException
     */
    public PaginatedList queryForPaginatedList(String string, Object object, int _int)
        throws DAOException {
    	
        try {
            return sqlMapClient.queryForPaginatedList(string, object, _int);
        } catch(SQLException ex) {
            throw createException(ex, string, object);
        }
    }
        
    /**
     * queryWithRowHandler
     * @param string String
     * @param object Object
     * @param rowHandler RowHandler
     * @throws DAOException
     */
    public void queryWithRowHandler(String string, Object object, RowHandler rowHandler)
        throws DAOException {
    	
        try {
            sqlMapClient.queryWithRowHandler(string, object, rowHandler);
        } catch(SQLException ex) {
            throw createException(ex, string, object);
        }
    }

    /**
     * setUserConnection
     * @param connection Connection
     * @throws DAOException
     */
    public void setUserConnection(Connection connection)
        throws DAOException {
    	
        try {
            sqlMapClient.setUserConnection(connection);
        } catch(SQLException ex) {
            throw new DAOException(ex, KEY_USERCONNECTION);
        }
    }
        
    /**
     * startBatch
     * @throws DAOException
     */
    public void startBatch() throws DAOException {
    	
        try {
            sqlMapClient.startBatch();
        } catch(SQLException ex) {
            throw new DAOException(ex, KEY_STARTBATCH);
        }
    }
        
    /**
     * startTransaction
     * @throws DAOException
     */
    public void startTransaction() throws DAOException {
    	
        try {
            sqlMapClient.startTransaction();
        } catch(SQLException ex) {
            throw new DAOException(ex, KEY_BEGINTRANSACTION);
        }
    }

    /**
     * startTransaction
     * @param _int int
     * @throws DAOException
     */
    public void startTransaction(int _int) throws DAOException {
    	
        try {
            sqlMapClient.startTransaction(_int);
        } catch(SQLException ex) {
            throw new DAOException(ex, KEY_BEGINTRANSACTION);
        }
    }
        
    /**
     * update
     * @param string String
     * @param object Object
     * @return int
     * @throws DAOException
     */
    public int update(String string, Object object) throws DAOException {
    	
        try {
            return sqlMapClient.update(string, object);
        } catch(SQLException ex) {
            throw createException(ex, string, object);
        }
    }
        
    /**
     * Crea la excepcin que ser lanzada, usa exceptionTranslator si esta definido
     * @param e excepcin causante
     * @param queryName nombre de la consulta que se estaba ejecutando
     * @param params parametros de la consulta
     * @return Excepcin que ser lanzada.
     */
    private DAOException createException(Exception exception, String message, Object params){
        return new DAOException(exception, message, params);
    }

    /**
     * @return
     */
    public SqlMapClient getSqlMapClient() {
        return sqlMapClient;
    }

}
